'''A couple of logging handlers which should play nicely with the Python
logging library.'''
import logging
try:
    from simplejson import dumps
except ImportError:
    from json import dumps
from logging.handlers import SysLogHandler
from hoover.session import LogglySession
from hoover.utils import async_post_to_endpoint


class LogglyHttpHandler(logging.Handler):
    def __init__(self, session=None, token='', inputname='', input=None,
                 announce=False, json_class=None):
        logging.Handler.__init__(self)
        if inputname:
            # TODO: raise something appropriate if session is None
            input = session.get_input_by_name(inputname)
        if input:
            self.inputobj = input
            try:
                token = input.input_token
                self.inputname = input.name
            except AttributeError:
                raise ValueError('This is not an HTTP input')
        session = session or LogglySession
        self.token = token
        self.endpoint = "https://%s/inputs/%s" % (session.proxy, token)
        self.json_class = json_class
        # TODO: verify we can write to the input
        if announce:
            # TODO: grab this boxes' IP, and announce logging to the input
            pass

    def emit(self, record):
        if isinstance(record.msg, (list, dict)):
            record.msg = dumps(record.msg, cls=self.json_class)
        msg = self.format(record)
        async_post_to_endpoint(self.endpoint, msg)


class LogglySyslogHandler(SysLogHandler):
    def __init__(self, session=None, port=None, inputname='', input=None,
                 announce=False, authorize=True, **kwargs):
        #TODO: avoid duplication with __init__ above
        if inputname:
            # raise if no session
            input = session.get_input_by_name(inputname)
        if input:
            self.inputobj = input
            try:
                port = input.port
                self.inputname = input.name
            except AttributeError:
                raise ValueError("This doesn't look like a syslog input")
            if authorize:
                if port == 514:
                    # raise if no session
                    session._api_help('api/inputs/%s/add514' % input.id)
                else:
                    session._api_help('api/inputs/%s/adddevice' % input.id,
                                     method='POST')
        self.port = port
        session = session or LogglySession
        SysLogHandler.__init__(self, address=(session.proxy, port),
                               **kwargs)



class LogglyInput(object):
    def __init__(self, attributes, session):
        for attr in attributes.keys():
            setattr(self, attr, attributes[attr])
        self.session = session

    def __repr__(self):
        return "Input:%s" % self.name

    def get_handler(self, **kwargs):
        from hoover.handlers import LogglyHttpHandler, LogglySyslogHandler
        if self.service['name'] == 'HTTP':
            return LogglyHttpHandler(self.session, input=self, **kwargs)
        return LogglySyslogHandler(self.session, input=self, **kwargs)

    def search(self, q='', **kwargs):
        '''Thin wrapper on LogglySession.search, restricting the search to
        data from this input.'''
        q = '%s inputname:%s' % (q, self.name)
        return self.session.search(q, **kwargs)

    def facets(self, q='', **kwargs):
        '''Thin wrapper on LogglySession.facets, restricting the search to
        data from this input.'''
        q = '%s inputname:%s' % (q, self.name)
        return self.session.facets(q, **kwargs)

    def delete(self):
        '''Deletes the input from your Loggly account.'''
        self.session._api_help('api/inputs/%s' % self.id, method='DELETE')
        self.session.inputs.remove(self)

    def set_discover(self, state=True):
        '''Given state=True,False, puts the input into, or takes it out of
        discovery mode.'''
        method = state and 'POST' or 'DELETE'
        self.session._api_help('api/inputs/%d/discover/' % self.id,
                              method=method)
        self.discover = state



import hoover
import logging
from time import sleep

session = hoover.LogglySession('hoovertest', 'hoover', 'b34v3r5')
i = session.get_input_by_name('sysloginput2')
handler = i.get_handler()
logger = logging.getLogger('syslogtologgly')
logger.addHandler(handler)
handler.setLevel(logging.DEBUG)
logger.setLevel(logging.DEBUG)
logger.info('this will go straight to loggly')

sleep(10)

print i.search(starttime='NOW-2MINUTES')



from functools import wraps
from httplib2 import Http


def async(func):
    '''Awesome decorator for asyncronizing functions.
    Don't use this if you care about return value.'''
    from threading import Thread

    class FuncRunner(Thread):
        def __init__(self, args, kwargs):
            super(FuncRunner, self).__init__()
            self.args = args
            self.kwargs = kwargs

        def run(self):
            func(*(self.args), **(self.kwargs))

    def newfunc(*args, **kwargs):
        FuncRunner(args, kwargs).start()

    return wraps(func)(newfunc)


def post_to_endpoint(endpoint, message):
    h = Http()
    h.request(endpoint, 'POST', message)
async_post_to_endpoint = async(post_to_endpoint)


def time_translate(func):
    def new_func(*args, **kwargs):
        if 'starttime' in kwargs:
            kwargs['from'] = kwargs.pop('starttime')
        if 'endtime' in kwargs:
            kwargs['until'] = kwargs.pop('endtime')
        return func(*args, **kwargs)
    return wraps(func)(new_func)



from __future__ import print_function
from hoover.input import LogglyInput
from hoover.exceptions import NotFound, AuthFail
from hoover.utils import time_translate
import logging
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode
from httplib2 import Http
try:
    from simplejson import loads
except ImportError:
    from json import loads


class LogglySession(object):
    domain = 'loggly.com'
    proxy = 'logs.loggly.com'

    def __init__(self, subdomain, username, password, domain=None, proxy=None,
                 secure=True):
        '''pass in subdomain, username, and password to authorize all API
        transactions.'''
        self.subdomain = subdomain
        self.username = username
        self.password = password
        if domain:
            # Mostly for internal testing
            # Yes, there are other Logglys =P
            self.domain = domain
        if proxy:
            self.proxy = proxy
        self.protocol = secure and 'https' or 'http'

    def _api_help(self, endpoint, params=None, method='GET'):
        h = Http()
        h.add_credentials(self.username, self.password)
        url = '%s://%s.%s/%s' % (self.protocol, self.subdomain, self.domain,
                                 endpoint)
        if method == 'GET':
            body = ''
            if params:
                url += '?' + urlencode(params)
        elif params:
            body = urlencode(params)
        else:
            body = ''
        headers, results = h.request(url, method, body)
        status = headers['status']
        if int(status) == 401:
            raise AuthFail('Sorry, your authentication was not accepted.')
        # TODO check status, raise appropriate errors or something
        try:
            return loads(results.decode('utf-8'))
        except ValueError:
            return results

    @property
    def inputs(self):
        if not hasattr(self, '_inputs'):
            self._inputs_init()
        return self._inputs

    def _inputs_init(self):
        inputs = self._api_help('api/inputs')
        self._inputs = [LogglyInput(i, self) for  i in inputs]

    @property
    def http_inputs(self):
        return [i for i in self.inputs if i.service['name'] == 'HTTP']

    def get_input_by_name(self, name):
        '''Locates an input by name. Case insensitive.'''
        try:
            (result,) = [i for i in self.inputs
                         if i.name.lower() == name.lower()]
        except ValueError:
            raise NotFound('Input %s not found.' % name)
        return result

    def config_inputs(self):
        '''For each input in your loggly account, register a python logger
        with the input's name logging to the input.'''
        for input in self.inputs:
            logger = logging.getLogger(input.name)
            logger.addHandler(input.get_handler())

    @time_translate
    def search(self, q='*', **kwargs):
        '''Thin wrapper on Loggly's text search API. First parameter is a query
        string.'''
        kwargs['q'] = q
        return self._api_help('api/search', kwargs)

    @time_translate
    def facets(self, q='*', facetby='date', **kwargs):
        '''Thin wrapper on Loggly's facet search API. facetby can be input, ip,
        or a json parameter of the form json.foo'''
        kwargs['q'] = q
        return self._api_help('api/facets/%s' % facetby, kwargs)

    def create_input(self, name, service='syslogudp', description='',
                     json=False):
        '''Creates a new input on your loggly account. Service can be any of:
            syslogudp
            syslogtcp
            syslogudp_strip
            syslogtcp_strip
            syslog 514
            HTTP
            syslog_tls.
        JSON can only be used with HTTP inputs.'''
        if json and service.lower() != 'http':
            raise ValueError("only HTTP inputs can use JSON")
        format = json and 'json' or 'text'
        params = {'name': name, 'service': service, 'description': description,
                  'format': format}
        result = self._api_help('api/inputs', params, method='POST')
        try:
            newinput = LogglyInput(result, self)
        except:
            #TODO
            raise
        self.inputs.append(newinput)
        return newinput



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='Hoover',
      version='0.5.1',
      description="Library for logging to Loggly from within Python webapps",
      author="Mike Blume",
      author_email="mike@loggly.com",
      url="http://www.github.com/loggly/hoover",
      packages=['hoover'],
      install_requires=['httplib2'],
)
